class TypingRelations:
    instance_of = 'P31'
    subclass_of = 'P279'


class People:
    gender = 'P21'

    place_of_birth = 'P19'
    place_of_death = 'P20'

    date_of_birth = 'P569'
    date_of_death = 'P570'

    native_lang = 'P103'
    lang_spoken = 'P1412'
    lang_written = 'P6886'

    occupation = 'P106'
    ethnic_group = 'P172'

    influenced_by = 'P737'
    field_of_work = 'P101'
    educated_at = 'P69'
    notable_work = 'P800'


class WrittenWorks:
    publication_date = 'P577'
    language = 'P407'
    country_of_origin = 'P495'
    main_sub = 'P921'
    genre = 'P136'
    author = 'P50'


class Location:
    country = 'P17'
    coordinate = 'P625'
