# SPDX-FileCopyrightText: 2024-present Hao Wu <haowu@dataset.sh>
#
# SPDX-License-Identifier: MIT
from .db import auto_connect, connect
from .labels import lookup_label
from .wiki_link import lookup_wikilink